from typing import Any, List, Optional

from fastapi import APIRouter, Depends, HTTPException
from sqlalchemy.orm import Session

from app import crud, models, schemas
from app.api import deps

router = APIRouter()


@router.get("/", response_model=List[schemas.Product])
def read_products(
    db: Session = Depends(deps.get_db),
    skip: int = 0,
    limit: int = 100,
    name: Optional[str] = None,
    category_id: Optional[int] = None,
    supplier_id: Optional[int] = None,
    active_only: bool = False,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Retrieve products with optional filtering.
    """
    if name:
        products = crud.product.get_by_name(db, name=name)
    elif category_id:
        products = crud.product.get_by_category_id(db, category_id=category_id, skip=skip, limit=limit)
    elif supplier_id:
        products = crud.product.get_by_supplier_id(db, supplier_id=supplier_id, skip=skip, limit=limit)
    elif active_only:
        products = crud.product.get_active(db, skip=skip, limit=limit)
    else:
        products = crud.product.get_multi(db, skip=skip, limit=limit)
    return products


@router.post("/", response_model=schemas.Product)
def create_product(
    *,
    db: Session = Depends(deps.get_db),
    product_in: schemas.ProductCreate,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Create new product.
    """
    # Check if SKU exists
    product = crud.product.get_by_sku(db, sku=product_in.sku)
    if product:
        raise HTTPException(
            status_code=400,
            detail="A product with this SKU already exists in the system.",
        )
    
    # Check if barcode exists (if provided)
    if product_in.barcode:
        product = crud.product.get_by_barcode(db, barcode=product_in.barcode)
        if product:
            raise HTTPException(
                status_code=400,
                detail="A product with this barcode already exists in the system.",
            )
    
    # Validate category_id if provided
    if product_in.category_id:
        category = crud.category.get(db, id=product_in.category_id)
        if not category:
            raise HTTPException(
                status_code=404,
                detail=f"Category with ID {product_in.category_id} not found",
            )
    
    # Validate supplier_id if provided
    if product_in.supplier_id:
        supplier = crud.supplier.get(db, id=product_in.supplier_id)
        if not supplier:
            raise HTTPException(
                status_code=404,
                detail=f"Supplier with ID {product_in.supplier_id} not found",
            )
    
    product = crud.product.create(db, obj_in=product_in)
    
    # Create initial inventory record with 0 quantity
    inventory_in = schemas.InventoryCreate(
        product_id=product.id,
        quantity=0,
        location=None
    )
    crud.inventory.create(db, obj_in=inventory_in)
    
    return product


@router.get("/{id}", response_model=schemas.Product)
def read_product(
    *,
    db: Session = Depends(deps.get_db),
    id: int,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Get product by ID.
    """
    product = crud.product.get(db, id=id)
    if not product:
        raise HTTPException(status_code=404, detail="Product not found")
    return product


@router.get("/sku/{sku}", response_model=schemas.Product)
def read_product_by_sku(
    *,
    db: Session = Depends(deps.get_db),
    sku: str,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Get product by SKU.
    """
    product = crud.product.get_by_sku(db, sku=sku)
    if not product:
        raise HTTPException(status_code=404, detail="Product not found")
    return product


@router.get("/barcode/{barcode}", response_model=schemas.Product)
def read_product_by_barcode(
    *,
    db: Session = Depends(deps.get_db),
    barcode: str,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Get product by barcode.
    """
    product = crud.product.get_by_barcode(db, barcode=barcode)
    if not product:
        raise HTTPException(status_code=404, detail="Product not found")
    return product


@router.put("/{id}", response_model=schemas.Product)
def update_product(
    *,
    db: Session = Depends(deps.get_db),
    id: int,
    product_in: schemas.ProductUpdate,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Update a product.
    """
    product = crud.product.get(db, id=id)
    if not product:
        raise HTTPException(status_code=404, detail="Product not found")
    
    # Check if SKU exists and is different from current product
    if product_in.sku and product_in.sku != product.sku:
        existing_product = crud.product.get_by_sku(db, sku=product_in.sku)
        if existing_product:
            raise HTTPException(
                status_code=400,
                detail="A product with this SKU already exists in the system.",
            )
    
    # Check if barcode exists and is different from current product
    if product_in.barcode and product_in.barcode != product.barcode:
        existing_product = crud.product.get_by_barcode(db, barcode=product_in.barcode)
        if existing_product:
            raise HTTPException(
                status_code=400,
                detail="A product with this barcode already exists in the system.",
            )
    
    # Validate category_id if provided
    if product_in.category_id and product_in.category_id != product.category_id:
        category = crud.category.get(db, id=product_in.category_id)
        if not category:
            raise HTTPException(
                status_code=404,
                detail=f"Category with ID {product_in.category_id} not found",
            )
    
    # Validate supplier_id if provided
    if product_in.supplier_id and product_in.supplier_id != product.supplier_id:
        supplier = crud.supplier.get(db, id=product_in.supplier_id)
        if not supplier:
            raise HTTPException(
                status_code=404,
                detail=f"Supplier with ID {product_in.supplier_id} not found",
            )
    
    product = crud.product.update(db, db_obj=product, obj_in=product_in)
    return product


@router.delete("/{id}", status_code=204, response_model=None)
def delete_product(
    *,
    db: Session = Depends(deps.get_db),
    id: int,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Delete a product.
    """
    product = crud.product.get(db, id=id)
    if not product:
        raise HTTPException(status_code=404, detail="Product not found")
    
    # First, delete associated inventory records
    inventory = crud.inventory.get_by_product_id(db, product_id=id)
    if inventory:
        db.delete(inventory)
        db.commit()
    
    crud.product.remove(db, id=id)
    return None